unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls;

type

  { TForm1 }

  TRotation = (rUndefined, rClockwise, rCounterClockwise);

  TForm1 = class(TForm)
    PaintBox1: TPaintBox;
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintBox1Paint(Sender: TObject);
    procedure PaintBox1Resize(Sender: TObject);
  private
    { private declarations }
    FStartAngle, FEndAngle: Double;
    FRotation: TRotation;
    xc, yc, a, b: Integer;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

function DegToRad(Angle: Double): Double;
begin
  Result := Angle * pi / 180;
end;

procedure Exchange(var a, b: Double);
var
  tmp: Double;
begin
  tmp := a;
  a := b;
  b := tmp;
end;

procedure DrawEllipse(ACanvas: TCanvas; xc,yc: Integer; a,b: Integer;
  StartAngle,EndAngle: Double; ARotation: TRotation; AFillColor: TColor);
const
  delta_angle = 1.0;
var
  angle: double;
  points: Array of TPoint;
  npoints: Integer;
  done: Boolean;
begin
  if ARotation = rCounterClockwise then
    Exchange(StartAngle, EndAngle);

  if StartAngle > EndAngle then
    StartAngle := StartAngle - 360;

  npoints := 0;
  SetLength(points, 1000);
  points[0] := Point(xc, yc);  // center point
  angle := StartAngle;
  nPoints := 1;
  done := false;
  while not done do begin
    if ARotation <> rUndefined then
      angle := angle + delta_angle;
    if angle >= EndAngle then begin
      done := true;
      angle := EndAngle;
    end;
    points[nPoints] := Point(xc + round(a*cos(DegToRad(angle))), yc + round(b*sin(DegToRad(angle))));
    inc(nPoints);
    if nPoints > Length(points) then SetLength(points, Length(points) + 1000);
  end;
  points[nPoints] := Point(xc, yc);
  inc(nPoints);
  SetLength(points, npoints);

  with ACanvas do begin
    Pen.Style := psSolid;
    Pen.Color := clBlack;
    Brush.Color := AFillColor;
    Brush.Style := bsSolid;
    Polygon(@points[0], npoints);
  end;
end;


function QuadrantAngle(X,Y,xm,ym: Double): Double;
// Könnte noch etwas vereinfacht werden...
begin
  if (X = xm) then
  begin
    if (Y > ym) then
      Result := 90
    else
    if (Y < ym) then
      Result := 270
    else
       raise Exception.Create('0/0 ist undefiniert');
  end else
  begin
    Result := arctan(abs((Y - ym) / (X - xm))) * 180 / pi;
    // Das ist der Winkel im 1. Quadranten, ist immer >0!

    // 1. Quadrant  --> keine Korrektur nötig
    if (X > xm) and (Y >= ym) then
      exit;

    // 2. Quadrant  --> den Betrag des Winkels von 180° subtrahieren
    if (X < xm) and (Y >= ym) then
    begin
      Result := 180 - Result;
      exit;
    end;

    // 3.Quadrant --> entgegengesetzt um 1.Quadrant, also 180 addieren
    if (X < xm) and (Y <ym) then
    begin
      Result := 180 + Result;
      exit;
    end;

    // 4. Quadrant --> den Betrag des Winkels von 360° subtrahieren
    if (X > xm) and (Y < ym) then
    begin
      Result := 360 - Result;
      exit;
    end;
  end;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
var
  R: TRect;
begin
  Paintbox1.Canvas.Brush.Style := bsSolid;
  Paintbox1.Canvas.Brush.Color := clWhite;
  Paintbox1.Canvas.FillRect(0, 0, Paintbox1.Width, Paintbox1.Height);

  Paintbox1.Canvas.Pen.Style := psDot;
  Paintbox1.Canvas.Pen.Color := clSilver;
  Paintbox1.Canvas.Brush.Style := bsClear;
  Paintbox1.Canvas.Rectangle(xc-a, yc-b, xc+a, yc+b);

  if not ((FStartAngle = 0) and (FEndAngle = 0)) then
    DrawEllipse(Paintbox1.Canvas, xc, yc, a, b, FStartAngle, FEndAngle, FRotation, clRed);
end;

procedure TForm1.PaintBox1Resize(Sender: TObject);
begin
  xc := Paintbox1.Width div 2;
  yc := Paintbox1.Height div 2;
  a := round(0.75*xc);
  b := round(0.75*yc);
end;

procedure TForm1.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  dx, dy: Integer;
begin
  dx := X - xc;
  dy := Y - yc;
  FStartAngle := QuadrantAngle(dx*b + xc, dy*a + yc, xc, yc);
  FEndAngle := FStartAngle;
  FRotation := rUndefined;
  Paintbox1.Invalidate;
end;

procedure TForm1.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  dx, dy: Integer;
begin
  if Shift = [ssLeft] then begin
    dx := X - xc;
    dy := Y - yc;
    FEndAngle := QuadrantAngle(dx*b + xc, dy*a + yc, xc, yc);
    if FRotation = rUndefined then
    begin
      if FEndAngle > FStartAngle then
        FRotation := rClockwise else
        FRotation := rCounterClockwise;
    end;
    Paintbox1.Invalidate;
  end;
end;


end.

